"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var nats_1 = require("nats");
var pubsub_async_iterator_1 = require("./pubsub-async-iterator");
var NatsPubSub = (function () {
    function NatsPubSub(options) {
        if (options === void 0) { options = {}; }
        this.triggerTransform = options.triggerTransform || (function (trigger) { return trigger; });
        if (options.client) {
            this.natsConnection = options.client;
        }
        else {
            var brokerUrl = 'nats://127.0.0.1:4222';
            this.natsConnection = nats_1.connect(brokerUrl);
        }
        this.logger = options.logger.child({ className: 'nats-subscription' });
        if (options.connectionListener) {
            this.natsConnection.on('connect', options.connectionListener);
            this.natsConnection.on('error', options.connectionListener);
            this.natsConnection.on('disconnect', options.connectionListener);
            this.natsConnection.on('reconnecting', options.connectionListener);
            this.natsConnection.on('reconnect', options.connectionListener);
            this.natsConnection.on('close', options.connectionListener);
        }
        else {
            this.natsConnection.on('error', this.logger && this.logger.error);
        }
        this.subscriptionMap = {};
        this.subsRefsMap = {};
        this.natsSubMap = {};
        this.currentSubscriptionId = 0;
        this.publishOptionsResolver = options.publishOptions || (function () { return Promise.resolve({}); });
        this.subscribeOptionsResolver = options.subscribeOptions || (function () { return Promise.resolve({}); });
        this.parseMessageWithEncoding = options.parseMessageWithEncoding;
    }
    NatsPubSub.prototype.publish = function (trigger, payload) {
        this.logger && this.logger.trace("publishing for queue '%s' (%j)", trigger, payload);
        var message = Buffer.from(JSON.stringify(payload), this.parseMessageWithEncoding);
        this.natsConnection.publish(trigger, message);
        return true;
    };
    NatsPubSub.prototype.subscribe = function (trigger, onMessage, options) {
        var _this = this;
        this.logger && this.logger.trace("subscribing to queue '%s' with onMessage (%j), and options (%j) ", trigger, onMessage, options);
        var triggerName = this.triggerTransform(trigger, options);
        var id = this.currentSubscriptionId++;
        this.subscriptionMap[id] = [triggerName, onMessage];
        var refs = this.subsRefsMap[triggerName];
        if (refs && refs.length > 0) {
            this.logger.trace('refs already exist calling it (%j)', refs);
            var newRefs = refs.concat([id]);
            this.subsRefsMap[triggerName] = newRefs;
            this.logger.trace('returing subId (%s)', id);
            return Promise.resolve(id);
        }
        else {
            return new Promise(function (resolve, reject) {
                _this.subscribeOptionsResolver(trigger, options).then(function (subscriptionOptions) {
                    _this.logger.trace('resolve subscriptionoptions with options (%j)', options);
                    var subId = _this.natsConnection.subscribe(triggerName, function (msg) { return _this.onMessage(triggerName, msg); });
                    _this.subsRefsMap[triggerName] = (_this.subsRefsMap[triggerName] || []).concat([id]);
                    _this.natsSubMap[triggerName] = subId;
                    resolve(id);
                });
            });
        }
    };
    NatsPubSub.prototype.unsubscribe = function (subId) {
        var _a = (this.subscriptionMap[subId] || [])[0], triggerName = _a === void 0 ? null : _a;
        this.logger.trace('dumping saved subsMappers (%j)', this.subsRefsMap);
        var refs = this.subsRefsMap[triggerName];
        var natsSubId = this.natsSubMap[triggerName];
        this.logger && this.logger.trace("unsubscribing to queue '%s' and natsSid: (%s)", subId, natsSubId);
        if (!refs) {
            this.logger.error('there are no subscriptions for triggerName (%s) and natsSid (%s)', triggerName, natsSubId);
            throw new Error("There is no subscription of id \"" + subId + "\"");
        }
        if (refs.length === 1) {
            this.logger.trace("unsubscribing and there won't be any subscriber");
            this.natsConnection.unsubscribe(natsSubId);
            delete this.natsSubMap[triggerName];
            delete this.subsRefsMap[triggerName];
            this.logger.trace('unsubscribe on nats for subId (%s) is completed', natsSubId);
        }
        else {
            var index = refs.indexOf(subId);
            var newRefs = index === -1 ? refs : refs.slice(0, index).concat(refs.slice(index + 1));
            this.subsRefsMap[triggerName] = newRefs;
        }
        delete this.subscriptionMap[subId];
    };
    NatsPubSub.prototype.asyncIterator = function (triggers) {
        return new pubsub_async_iterator_1.PubSubAsyncIterator(this, triggers);
    };
    NatsPubSub.prototype.onMessage = function (topic, message) {
        this.logger.trace('triggered onMessage with topic (%s), message (%j)', topic, message);
        var subscribers = this.subsRefsMap[topic];
        if (!subscribers || !subscribers.length) {
            return;
        }
        var messageString = message.toString(this.parseMessageWithEncoding);
        var parsedMessage;
        try {
            parsedMessage = JSON.parse(messageString);
        }
        catch (e) {
            parsedMessage = messageString;
        }
        for (var _i = 0, subscribers_1 = subscribers; _i < subscribers_1.length; _i++) {
            var subId = subscribers_1[_i];
            var listener = this.subscriptionMap[subId][1];
            listener(parsedMessage);
        }
    };
    return NatsPubSub;
}());
exports.NatsPubSub = NatsPubSub;
//# sourceMappingURL=nats-pubsub.js.map